{
    Copyright (c) 2011 by Sven Barth

    Android application for managing train rides.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 ****************************************************************************
}

unit Dialogs;

{$namespace org.sven.trainlog}
{$mode objfpc}{$H+}
{$modeswitch unicodestrings}

interface

uses
  androidr14;

type
  TDialogNotifyEvent = function(aSender: AADialog): Boolean of object;

  TIDDialog = class(AADialog)
  private
    fID: Integer;
    fOnCancelClick: TDialogNotifyEvent;
    fOnOKClick: TDialogNotifyEvent;
  public
    property ID: Integer read fID write fID;
    property OnOKClick: TDialogNotifyEvent read fOnOKClick write fOnOKClick;
    property OnCancelClick: TDialogNotifyEvent read fOnCancelClick
      write fOnCancelClick;
  end;

  TNameDialog = class(TIDDialog, AVView.InnerOnClickListener)
  private
    function GetText: String;
    procedure SetText(aValue: String);
  public
    constructor create(aContext: ACContext);
    procedure onClick(aView: AVView);
    property Text: String read GetText write SetText;
  end;

  TNameValueDialog = class(TIDDialog, AVView.InnerOnClickListener)
  private
    function GetText: String;
    function GetValue: LongInt;
    procedure SetText(aValue: String);
    procedure SetValue(aValue: LongInt);
  public
    constructor create(aContext: ACContext);
    procedure onClick(aView: AVView);
    property Text: String read GetText write SetText;
    property Value: LongInt read GetValue write SetValue;
  end;

implementation

uses
  Resources;

{ TNameDialog }

constructor TNameDialog.Create(aContext: ACContext);
var
  button: AWButton;
begin
  inherited Create(aContext);
  setContentView(R.layout.dialog_name);
  button := AWButton(findViewById(R.id.dialog_name_ok));
  if Assigned(button) then
    button.setOnClickListener(Self);
  button := AWButton(findViewById(R.id.dialog_name_cancel));
  if Assigned(button) then
    button.setOnClickListener(Self);
end;

function TNameDialog.GetText: String;
var
  edittext: AWEditText;
begin
  edittext := AWEditText(findViewById(R.id.dialog_name_name));
  if Assigned(edittext) then
    Result := edittext.getText.toString
  else
    Result := '';
end;

procedure TNameDialog.onClick(aView: AVView);
begin
  case aView.getId of
    R.id.dialog_name_ok: begin
      if not Assigned(OnOkClick) or OnOkClick(Self) then
        dismiss;
    end;
    R.id.dialog_name_cancel: begin
      if not Assigned(OnCancelClick) or OnCancelClick(Self) then
        cancel;
    end;
  end;
end;

procedure TNameDialog.SetText(aValue: String);
var
  edittext: AWEditText;
begin
  edittext := AWEditText(findViewById(R.id.dialog_name_name));
  if Assigned(edittext) then
    edittext.setText(JLString(aValue));
end;

{ TNameValueDialog }

constructor TNameValueDialog.Create(aContext: ACContext);
var
  button: AWButton;
begin
  inherited Create(aContext);
  setContentView(R.layout.dialog_name_value);
  button := AWButton(findViewById(R.id.dialog_name_value_ok));
  if Assigned(button) then
    button.setOnClickListener(Self);
  button := AWButton(findViewById(R.id.dialog_name_value_cancel));
  if Assigned(button) then
    button.setOnClickListener(Self);
end;

function TNameValueDialog.GetText: String;
var
  edittext: AWEditText;
begin
  edittext := AWEditText(findViewById(R.id.dialog_name_value_name));
  if Assigned(edittext) then
    Result := edittext.getText.toString
  else
    Result := '';
end;

function TNameValueDialog.GetValue: LongInt;
var
  edittext: AWEditText;
begin
  edittext := AWEditText(findViewById(R.id.dialog_name_value_value));
  if Assigned(edittext) then
    Result := JLInteger.Create(edittext.getText.toString.trim).intValue
  else
    Result := 0;
end;

procedure TNameValueDialog.onClick(aView: AVView);
begin
  case aView.getId of
    R.id.dialog_name_value_ok: begin
      if not Assigned(OnOkClick) or OnOkClick(Self) then
        dismiss;
    end;
    R.id.dialog_name_value_cancel: begin
      if not Assigned(OnCancelClick) or OnCancelClick(Self) then
        cancel;
    end;
  end;
end;

procedure TNameValueDialog.SetText(aValue: String);
var
  edittext: AWEditText;
begin
  edittext := AWEditText(findViewById(R.id.dialog_name_value_name));
  if Assigned(edittext) then
    edittext.setText(JLString(aValue));
end;

procedure TNameValueDialog.SetValue(aValue: LongInt);
var
  edittext: AWEditText;
begin
  edittext := AWEditText(findViewById(R.id.dialog_name_value_value));
  if Assigned(edittext) then
    edittext.setText(JLInteger.Create(aValue).toString);
end;

end.

