{
    Copyright (c) 2011 by Sven Barth

    Android application for managing train rides.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 ****************************************************************************
}

unit Train;

{$mode objfpc}{$H+}
{$namespace org.sven.trainlog}
{$modeswitch unicodestrings}

interface

uses
  AndroidR14, Resources, TrainData;

type

  { TTrainActivity }

  TTrainActivity = class(AAListActivity,
    AWAdapterView.InnerOnItemClickListener,
    AWAdapterView.InnerOnItemLongClickListener,
    ACDialogInterface.InnerOnClickListener)
  private
    const
      DialogIDAdd = 1;
      DialogIDEdit = 2;
      DialogIDDelete = 3;
  private
    function HandleDialogOKClick(aSender: AADialog): Boolean;
  protected
    fAdapter: AWSimpleCursorAdapter;
    fDataHelper: TTrainLogHelper;
    fID: jlong;
    fSelectedID: jlong;
    procedure UpdateData(aFirst: Boolean);
  public
    procedure onCreate(savedInstanceState: AOBundle); override;
    function onCreateDialog(aID: jint): AADialog; override;
    procedure onPrepareDialog(aID: jint; aDialog: AADialog); override;
    procedure onClick(aDialog: ACDialogInterface; aWhich: jint);
    function onCreateOptionsMenu(aMenu: AVMenu): jboolean; override;
    function onOptionsItemSelected(aItem: AVMenuItem): jboolean; override;
    procedure onItemClick(aParent: AWAdapterView; aView: AVView;
      aPosition: jint; aID: jlong);
    function onItemLongClick(aParent: AWAdapterView; aView: AVView;
      aPosition: jint; aID: jlong): jboolean;
    procedure onCreateContextMenu(aMenu: AVContextMenu; aView: AVView;
      aMenuInfo: AVContextMenu.InnerContextMenuInfo); override;
    function onContextItemSelected(aMenuItem: AVMenuItem): jboolean; override;
  end;

implementation

uses
  Dialogs;

{ TTrainActivity }

function TTrainActivity.HandleDialogOKClick(aSender: AADialog): Boolean;
begin
  if not (aSender is TIDDialog) then
    Exit(False);

  Result := False;
  case TIDDialog(aSender).ID of
    DialogIDAdd: begin
      fDataHelper.AddTrain(TNameDialog(aSender).Text, fID);
      Result := True;
    end;
    DialogIDEdit: begin
      fDataHelper.NameOfTrain[fSelectedID] := TNameValueDialog(aSender).Text;
      fDataHelper.CountOfTrain[fSelectedID] := TNameValueDialog(aSender).Value;
      Result := True;
    end;
  end;

  if Result then
    UpdateData(False);
end;

procedure TTrainActivity.UpdateData(aFirst: Boolean);
var
  cur: ADCursor;
  colfrom: array[0..1] of JLString;
begin
  // swapCursor only exists since r11
{$ifndef android_r11}
  if aFirst then begin
{$endif}
  cur := fDataHelper.GetTrainsByType(fID);
  startManagingCursor(cur);

{$ifdef android_r11}
  if aFirst then begin
{$endif}
    colfrom[0] := TTrainLogHelper.ColumnTrainName;
    colfrom[1] := TTrainLogHelper.ColumnTrainCount;

    fAdapter := AWSimpleCursorAdapter.Create(Self,
                  R.layout.train_list_item, cur, colfrom,
                  [R.id.train_name, R.id.train_count]);

    setListAdapter(fAdapter);
  end else
{$ifdef android_r11}
    fAdapter.swapCursor(cur);
{$else}
    fAdapter.getCursor.requery;
{$endif}
end;

procedure TTrainActivity.onCreate(savedInstanceState: AOBundle);
begin
  inherited onCreate(savedInstanceState);

  fDataHelper := TTrainLogHelper.Create(Self);

  fID := getIntent.getLongExtra('org.sven.trainlog.ID', 0);
  if fID = 0 then
    raise ETrainLog.Create('No ID for train type given');

  setTitle(R.strings.name_trains);

  UpdateData(True);

  getListView.setOnItemClickListener(Self);
  getListView.setOnItemLongClickListener(Self);
  getListView.setOnCreateContextMenuListener(Self);
end;

function TTrainActivity.onCreateDialog(aID: jint): AADialog;
var
  builder: AAAlertDialog.InnerBuilder;
begin
  case aID of
    DialogIDAdd: begin
      Result := TNameDialog.Create(Self);
      TNameDialog(Result).OnOkClick := @HandleDialogOkClick;
      TNameDialog(Result).ID := DialogIDAdd;
      Result.setTitle(R.strings.title_train_add);
    end;
    DialogIDEdit: begin
      Result := TNameValueDialog.Create(Self);
      TNameValueDialog(Result).OnOkClick := @HandleDialogOkClick;
      TNameValueDialog(Result).ID := DialogIDEdit;
      Result.setTitle(R.strings.title_train_edit);
    end;
    DialogIDDelete: begin
      builder := AAAlertDialog.InnerBuilder.Create(Self);

      builder.setMessage(JLString('Are you sure you want to delete train ''' +
        fDataHelper.NameOfTrain[fSelectedID] + '''?'));

      builder.setPositiveButton(JLString('Yes'), Self);
      builder.setNegativeButton(JLString('No'), Self);

      Result := builder.create_;
    end;
    else
      Result := Nil;
  end;
end;

procedure TTrainActivity.onPrepareDialog(aID: jint; aDialog: AADialog);
begin
  if not (aDialog is TIDDialog) then
    Exit;

  case TIDDialog(aDialog).ID of
    DialogIDAdd:
      TNameDialog(aDialog).Text := '';
    DialogIDEdit: begin
      TNameValueDialog(aDialog).Text := fDataHelper.NameOfTrain[fSelectedID];
      TNameValueDialog(aDialog).Value := fDataHelper.CountOfTrain[fSelectedID];
    end;
  end;
end;

procedure TTrainActivity.onClick(aDialog: ACDialogInterface; aWhich: jint);
begin
  case aWhich of
    ACDialogInterface.BUTTON_NEGATIVE:
      aDialog.cancel;
    ACDialogInterface.BUTTON_POSITIVE: begin
      fDataHelper.DeleteTrain(fSelectedID);
      UpdateData(False);
      aDialog.dismiss;
    end;
  end;
end;

function TTrainActivity.onCreateOptionsMenu(aMenu: AVMenu): jboolean;
var
  inflater: AVMenuInflater;
begin
  inflater := getMenuInflater;
  inflater.inflate(R.menu.train_options, aMenu);
  Result := True;
end;

function TTrainActivity.onOptionsItemSelected(aItem: AVMenuItem):
  jboolean;
begin
  case aItem.getItemId of
    R.id.train_add: begin
      showDialog(DialogIDAdd);
      Result := True;
    end
    else
      Result := inherited;
  end;
end;

procedure TTrainActivity.onItemClick(aParent: AWAdapterView; aView: AVView;
  aPosition: jint; aID: jlong);
var
  s, s2: String;
  js: JLString;
  jl: JLLong;
begin
  if aParent.getContext <> Self then
    Exit;
  fSelectedID := aID;
  showDialog(DialogIDEdit);
end;

function TTrainActivity.onItemLongClick(aParent: AWAdapterView; aView: AVView;
  aPosition: jint; aID: jlong): jboolean;
var
  s: String;
  js: JLString;
begin
  if aParent.getContext <> Self then
    Exit(False);
  fSelectedID := aID;
  getListView.showContextMenu;
  Result := True;
end;

procedure TTrainActivity.onCreateContextMenu(aMenu: AVContextMenu;
  aView: AVView; aMenuInfo: AVContextMenu.InnerContextMenuInfo);
begin
  getMenuInflater.inflate(R.menu.train_context, aMenu);
end;

function TTrainActivity.onContextItemSelected(aMenuItem: AVMenuItem):
  jboolean;
begin
  case aMenuItem.getItemId of
    R.id.train_increase: begin
      fDataHelper.CountOfTrain[fSelectedID] :=
        fDataHelper.CountOfTrain[fSelectedID] + 1;
      UpdateData(False);
    end;
    R.id.train_edit:
      showDialog(DialogIDEdit);
    R.id.train_delete:
      showDialog(DialogIDDelete);
  end;
end;

end.
