{
    Copyright (c) 2011 by Sven Barth

    Android application for managing train rides.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 ****************************************************************************
}

unit TrainType;

{$mode objfpc}{$H+}
{$namespace org.sven.trainlog}
{$modeswitch unicodestrings}

interface

uses
  AndroidR14, Resources, TrainData;

type

  { TTrainTypeActivity }

  TTrainTypeActivity = class(AAListActivity,
    AWAdapterView.InnerOnItemClickListener,
    AWAdapterView.InnerOnItemLongClickListener,
    ACDialogInterface.InnerOnClickListener)
  private
    const
      DialogIDAdd = 1;
      DialogIDEdit = 2;
      DialogIDDelete = 3;
  private
    function HandleDialogOKClick(aSender: AADialog): Boolean;
  protected
    fAdapter: AWSimpleCursorAdapter;
    fDataHelper: TTrainLogHelper;
    fID: jlong;
    fSelectedID: jlong;
    procedure UpdateData(aFirst: Boolean);
  public
    procedure onCreate(savedInstanceState: AOBundle); override;
    function onCreateDialog(aID: jint): AADialog; override;
    procedure onPrepareDialog(aID: jint; aDialog: AADialog); override;
    procedure onClick(aDialog: ACDialogInterface; aWhich: jint);
    function onCreateOptionsMenu(aMenu: AVMenu): jboolean; override;
    function onOptionsItemSelected(aItem: AVMenuItem): jboolean; override;
    procedure onItemClick(aParent: AWAdapterView; aView: AVView;
      aPosition: jint; aID: jlong);
    function onItemLongClick(aParent: AWAdapterView; aView: AVView;
      aPosition: jint; aID: jlong): jboolean;
    procedure onCreateContextMenu(aMenu: AVContextMenu; aView: AVView;
      aMenuInfo: AVContextMenu.InnerContextMenuInfo); override;
    function onContextItemSelected(aMenuItem: AVMenuItem): jboolean; override;
  end;

implementation

uses
  Train, Dialogs;

{ TTrainTypeActivity }

function TTrainTypeActivity.HandleDialogOKClick(aSender: AADialog): Boolean;
begin
  if not (aSender is TNameDialog) then
    Exit(False);

  Result := False;
  case TNameDialog(aSender).ID of
    DialogIDAdd: begin
      fDataHelper.AddTrainType(TNameDialog(aSender).Text, fID);
      Result := True;
    end;
    DialogIDEdit: begin
      fDataHelper.NameOfTrainType[fSelectedID] := TNameDialog(aSender).Text;
      Result := True;
    end;
  end;

  if Result then
    UpdateData(False);
end;

procedure TTrainTypeActivity.UpdateData(aFirst: Boolean);
var
  cur: ADCursor;
  colfrom: array[0..0] of JLString;
begin
  // swapCursor only exists since r11
{$ifndef android_r11}
  if aFirst then begin
{$endif}
  cur := fDataHelper.GetTrainTypesByFamily(fID);
  startManagingCursor(cur);

{$ifdef android_r11}
  if aFirst then begin
{$endif}
    colfrom[0] := TTrainLogHelper.ColumnTrainTypeName;

    fAdapter := AWSimpleCursorAdapter.Create(Self,
                  R.layout.traintype_list_item, cur, colfrom,
                  [R.id.traintype_name]);

    setListAdapter(fAdapter);
  end else
{$ifdef android_r11}
    fAdapter.swapCursor(cur);
{$else}
    fAdapter.getCursor.requery;
{$endif}
end;

procedure TTrainTypeActivity.onCreate(savedInstanceState: AOBundle);
begin
  inherited onCreate(savedInstanceState);

  fDataHelper := TTrainLogHelper.Create(Self);

  fID := getIntent.getLongExtra('org.sven.trainlog.ID', 0);
  if fID = 0 then
    raise ETrainLog.Create('No ID for train type given');

  setTitle(R.strings.name_traintypes);

  UpdateData(True);

  getListView.setOnItemClickListener(Self);
  getListView.setOnItemLongClickListener(Self);
  getListView.setOnCreateContextMenuListener(Self);
end;

function TTrainTypeActivity.onCreateDialog(aID: jint): AADialog;
var
  builder: AAAlertDialog.InnerBuilder;
begin
  case aID of
    DialogIDAdd: begin
      Result := TNameDialog.Create(Self);
      TNameDialog(Result).OnOkClick := @HandleDialogOkClick;
      TNameDialog(Result).ID := DialogIDAdd;
      Result.setTitle(R.strings.title_traintype_add);
    end;
    DialogIDEdit: begin
      Result := TNameDialog.Create(Self);
      TNameDialog(Result).OnOkClick := @HandleDialogOkClick;
      TNameDialog(Result).ID := DialogIDEdit;
      Result.setTitle(R.strings.title_traintype_edit);
    end;
    DialogIDDelete: begin
      builder := AAAlertDialog.InnerBuilder.Create(Self);

      builder.setMessage(JLString('Are you sure you want to delete train ' +
        'type ''' + fDataHelper.NameOfTrainType[fSelectedID] + '''?'));

      builder.setPositiveButton(JLString('Yes'), Self);
      builder.setNegativeButton(JLString('No'), Self);

      Result := builder.create_;
    end;
    else
      Result := Nil;
  end;
end;

procedure TTrainTypeActivity.onPrepareDialog(aID: jint; aDialog: AADialog);
begin
  if not (aDialog is TNameDialog) then
    Exit;

  case TNameDialog(aDialog).ID of
    DialogIDAdd:
      TNameDialog(aDialog).Text := '';
    DialogIDEdit:
      TNameDialog(aDialog).Text := fDataHelper.NameOfTrainType[fSelectedID];
  end;
end;

procedure TTrainTypeActivity.onClick(aDialog: ACDialogInterface; aWhich: jint);
begin
  case aWhich of
    ACDialogInterface.BUTTON_NEGATIVE:
      aDialog.cancel;
    ACDialogInterface.BUTTON_POSITIVE: begin
      fDataHelper.DeleteTrainType(fSelectedID);
      UpdateData(False);
      aDialog.dismiss;
    end;
  end;
end;

function TTrainTypeActivity.onCreateOptionsMenu(aMenu: AVMenu): jboolean;
var
  inflater: AVMenuInflater;
begin
  inflater := getMenuInflater;
  inflater.inflate(R.menu.traintype_options, aMenu);
  Result := True;
end;

function TTrainTypeActivity.onOptionsItemSelected(aItem: AVMenuItem):
  jboolean;
begin
  case aItem.getItemId of
    R.id.traintype_add:
      showDialog(DialogIDAdd);
    else
      inherited;
  end;
end;

procedure TTrainTypeActivity.onItemClick(aParent: AWAdapterView; aView: AVView;
  aPosition: jint; aID: jlong);
var
  intent: ACIntent;
begin
  if aParent.getContext <> Self then
    Exit;
  intent := ACIntent.Create(Self, JLClass(TTrainActivity));
  intent.putExtra('org.sven.trainlog.ID', aID);
  startActivity(intent);
end;

function TTrainTypeActivity.onItemLongClick(aParent: AWAdapterView; aView: AVView;
  aPosition: jint; aID: jlong): jboolean;
var
  s: String;
  js: JLString;
begin
  if aParent.getContext <> Self then
    Exit(False);
  fSelectedID := aID;
  getListView.showContextMenu;
  Result := True;
end;

procedure TTrainTypeActivity.onCreateContextMenu(aMenu: AVContextMenu;
  aView: AVView; aMenuInfo: AVContextMenu.InnerContextMenuInfo);
begin
  getMenuInflater.inflate(R.menu.traintype_context, aMenu);
end;

function TTrainTypeActivity.onContextItemSelected(aMenuItem: AVMenuItem):
  jboolean;
begin
  case aMenuItem.getItemId of
    R.id.traintype_edit:
      showDialog(DialogIDEdit);
    R.id.traintype_delete:
      showDialog(DialogIDDelete);
  end;
end;

end.
